<?php declare(strict_types=1);
/**
 * Copyright © 2022 Amazon. All rights reserved.
 */

namespace Amazonfulfillment\AmazonMCF\Block\Adminhtml\Items\Column;

use Magento\Framework\App\ObjectManager;
use Magento\Framework\Filter\TruncateFilter\Result;
use Magento\Catalog\Helper\Data as CatalogHelper;
use Magento\Backend\Block\Template\Context;
use Magento\CatalogInventory\Api\StockRegistryInterface;
use Magento\CatalogInventory\Api\StockConfigurationInterface;
use Magento\Framework\Registry;
use Magento\Catalog\Model\Product\OptionFactory;
use Magento\Catalog\Helper\Data;

class Name extends \Magento\Sales\Block\Adminhtml\Items\Column\DefaultColumn
{
    /**
     * @var Result
     */
    private $truncateResult = null;
    /**
     * @var Context
     */
    private $context;
    /**
     * @var StockRegistryInterface
     */
    protected $stockRegistry;
    /**
     * @var StockConfigurationInterface
     */
    protected $stockConfiguration;
    /**
     * @var Registry
     */
    private $registry;
    /**
     * @var OptionFactory
     */
    private $optionFactory;
    /**
     * @var Data
     */
    private $helper;
    /**
     * @var CatalogHelper
     */
    private $catalogHelper;

    /**
     * @param Context $context
     * @param StockRegistryInterface $stockRegistry
     * @param StockConfigurationInterface $stockConfiguration
     * @param Registry $registry
     * @param OptionFactory $optionFactory
     * @param Data $helper
     * @param CatalogHelper $catalogHelper
     * @param array $data
     */
    public function __construct(
        Context $context,
        StockRegistryInterface $stockRegistry,
        StockConfigurationInterface $stockConfiguration,
        Registry $registry,
        OptionFactory $optionFactory,
        Data $helper,
        CatalogHelper $catalogHelper,
        array $data = []
    ) {
        $data['catalogHelper'] = $catalogHelper ?? ObjectManager::getInstance()->get(CatalogHelper::class);
        parent::__construct(
            $context,
            $stockRegistry,
            $stockConfiguration,
            $registry,
            $optionFactory,
            $data
        );
        $this->helper = $helper;
    }

    /**
     * Get Truncate string
     *
     * @param string $value
     * @param int $length
     * @param string $etc
     * @param string $remainder
     * @param bool $breakWords
     * @return string
     */
    public function truncateString(
        $value,
        $length = 80,
        $etc = '...',
        $remainder = '',
        $breakWords = true
    ) {
        $this->truncateResult = $this->filterManager->truncateFilter(
            $value,
            ['length' => $length, 'etc' => $etc, 'breakWords' => $breakWords]
        );
        return $this->truncateResult->getValue();
    }

    /**
     * Add line breaks and truncate value
     *
     * @param string $value
     * @return array
     */
    public function getFormattedOption($value)
    {
        $remainder = '';
        $this->truncateString($value, 55, '', $remainder);
        $result = [
            'value' => nl2br($this->truncateResult->getValue()),
            'remainder' => nl2br($this->truncateResult->getRemainder())
        ];
        return $result;
    }
    
    /**
     * Get Split SKUs
     *
     * @param string $value
     * @return array
     */
    public function getSplitSku($value)
    {
        return $this->helper->splitSku($value);
    }
}
