<?php
declare(strict_types=1);

/**
 * Copyright © 2022 Amazon. All rights reserved.
 */

namespace Amazonfulfillment\AmazonMCF\Block\Adminhtml\ShippingRate;

use Magento\Backend\Block\Template;
use Magento\Store\Model\StoreRepository;
use Magento\Backend\Block\Template\Context;
use Amazonfulfillment\AmazonMCF\Model\ConfigManager;
use Amazonfulfillment\AmazonMCF\Model\Config\Source\SyncStatus;
use Amazonfulfillment\AmazonMCF\Helper\Data;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Directory\Model\CurrencyFactory;

class Tabs extends Template
{
    /**
     * @var $_template
     */
    protected $_template = 'shipment/shipment_rates_tabs.phtml';
    /**
     * @var string
     */
    protected $_nameInLayout = 'shipment_rates_tabs';
    /**
     * @var StoreRepository
     */
    protected $storeRepository;
    /**
     * @param Context $context
     * @param StoreRepository $storeRepository
     * @param ConfigManager $configManager
     * @param SyncStatus $syncStatus
     * @param Data $helper
     * @param StoreManagerInterface $storeManager
     * @param CurrencyFactory $currencyFactory
     * @param array $data
     */
    public function __construct(
        Context $context,
        StoreRepository $storeRepository,
        ConfigManager $configManager,
        SyncStatus $syncStatus,
        Data $helper,
        StoreManagerInterface $storeManager,
        CurrencyFactory $currencyFactory,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->storeRepository = $storeRepository;
        $this->configManager = $configManager;
        $this->syncStatus =  $syncStatus;
        $this->helper = $helper;
        $this->storeManager = $storeManager;
        $this->currency = $currencyFactory->create();
    }
    /**
     * Return array of active websites
     */
    public function getWebsites()
    {
        return $this->helper->getActiveConnections();
    }
    /**
     * Return config value
     *
     * @param string $path
     * @param int $scopeId
     * @return mixed
     */
    public function getConfigData($path, $scopeId)
    {
        return $this->configManager->getConfigFromScope($path, $scopeId);
    }
     /**
      * Return array of Sync intervel
      *
      * @return array
      */
    public function getOptions()
    {
        return $this->syncStatus->getOptionArray();
    }
    /**
     * Get country code
     *
     * @return string
     */
    protected function getCountry()
    {
        return $this->configManager->getConfigCountry();
    }
    /**
     * Get config path
     *
     * @param string $path
     * @return string
     */
    public function getConfigPath($path)
    {
        if ($path == 'shipment_sync_status') {
            return 'amazonfulfillment/shipmentstatus/shipment_sync_status';
        } elseif ($path == 'shipment_sync_interval') {
            return 'amazonfulfillment/shipmentstatus/shipment_sync_interval';
        }
        return "";
    }
    /**
     * Return array of shipping rate options
     *
     * @param string $currencyCode
     * @return array
     */
    public function getRateOptions($currencyCode)
    {
        $ratesDetails = [
                            ['Standard','standard'],
                            ['Expedited','expedited'],
                            ['Priority','priority']
                        ];
        $rates = $this->deliveryOptions();
        if (isset($rates[$currencyCode])) {
            return $rates[$currencyCode];
        }
        return $ratesDetails;
    }
    /**
     * Get currency symbol by website id
     *
     * @param int $websiteId
     * @return string
     */
    public function getCurrencySymbol(int $websiteId)
    {
        $storeId = $this->storeManager->getWebsite($websiteId)->getDefaultStore()->getId();
        $currencyCode = $this->storeManager->getStore($storeId)->getDefaultCurrencyCode();
        return $this->currency->load($currencyCode)->getCurrencySymbol();
    }
    /**
     * Get Delivery Options
     *
     * @return array
     */
    public function deliveryOptions()
    {
        $options = [
            'US' => [
                        ['Standard (3-5 days)','standard'],
                        ['Expedited (2 days)','expedited'],
                        ['Priority (1 day)','priority']
                    ],
            'CA' => [
                        ['Standard (7 days)','standard'],
                        ['Expedited (3 days)','expedited'],
                        ['Priority (2 days)','priority']
                    ],
            'MX' => [
                        ['Standard (6 days)','standard'],
                        ['Expedited (2 days)','expedited'],
                        ['Priority','priority']
                    ],
            'GB' => [
                        ['Standard (3 days)','standard'],
                        ['Expedited (1 day)','expedited'],
                        ['Priority','priority']
                    ],
            'DE' => [
                        ['Standard (3 days)','standard'],
                        ['Expedited (1 day)','expedited'],
                        ['Priority','priority']
                    ],
            'FR' => [
                        ['Standard (4 days)','standard'],
                        ['Expedited (1 day)','expedited'],
                        ['Priority','priority']
                    ],
            'IT' => [
                        ['Standard (4 days)','standard'],
                        ['Expedited (1 day)','expedited'],
                        ['Priority','priority']
                    ],
            'ES' => [
                        ['Standard (4 days)','standard'],
                        ['Expedited (1 day)','expedited'],
                        ['Priority','priority']
                    ],
            'JP' => [
                        ['Standard (3 days)','standard'],
                        ['Expedited (2 days)','expedited'],
                        ['Priority (Scheduled)','priority']
                    ],
            'AU' => [
                        ['Standard (8 days)','standard'],
                        ['Expedited (7 days)','expedited'],
                        ['Priority (4 days)','priority']
                    ]
        ];
        return $options;
    }
}
