<?php
declare(strict_types=1);

/**
 * Copyright © 2022 Amazon. All rights reserved.
 */
namespace Amazonfulfillment\AmazonMCF\Controller\Adminhtml;

use Magento\Backend\App\Action\Context;
use Magento\Framework\Registry;
use Magento\Framework\View\Result\LayoutFactory;
use Psr\Log\LoggerInterface;
use Amazonfulfillment\AmazonMCF\Model\ConfigManager;

abstract class ShipmentStatus extends \Magento\Backend\App\Action
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;
    /**
     * @var Registry
     */
    private $coreRegistry;
    /**
     * @var LayoutFactory
     */
    private $resultLayoutFactory;
    /**
     * @var ConfigManager
     */
    public $configManager;
    /**
     * @param Context $context
     * @param Registry $coreRegistry
     * @param LayoutFactory $resultLayoutFactory
     * @param LoggerInterface $logger
     * @param ConfigManager $configManager
     */
    public function __construct(
        Context $context,
        Registry $coreRegistry,
        LayoutFactory $resultLayoutFactory,
        LoggerInterface $logger,
        ConfigManager $configManager
    ) {
        parent::__construct($context);
        $this->_coreRegistry = $coreRegistry;
        $this->_resultLayoutFactory = $resultLayoutFactory;
        $this->logger = $logger;
        $this->configManager = $configManager;
    }
    /**
     * Allowed Status
     *
     * @return int
     */
    protected function _isAllowed()
    {
        return $this->_authorization->isAllowed('Amazonfulfillment_AmazonMCF::shipment_status');
    }
    /**
     * Connected Status
     *
     * @return int
     */
    protected function isConnected()
    {
        return $this->configManager->isConnected();
    }
    /**
     * Get Country
     *
     * @return string
     */
    protected function getCountry()
    {
        return $this->configManager->getConfigCountry();
    }
}
