<?php declare(strict_types=1);
/**
 * Copyright © 2022 Amazon. All rights reserved.
 */
namespace Amazonfulfillment\AmazonMCF\Controller\Adminhtml\SkuMapping;

use Magento\Backend\App\Action\Context;
use Magento\Framework\Registry;
use Magento\Framework\View\Result\LayoutFactory;
use Magento\Framework\View\Result\PageFactory;
use Amazonfulfillment\AmazonMCF\Model\ConfigManager;
use Psr\Log\LoggerInterface;
use Amazonfulfillment\AmazonMCF\Model\McfLogManager;
use Amazonfulfillment\AmazonMCF\Helper\Data;
use Amazonfulfillment\AmazonMCF\Controller\Adminhtml\SkuMapping as Skumapping;
use Magento\Framework\MessageQueue\PublisherInterface;
use Magento\Framework\Serialize\Serializer\Json;

class AutoMap extends Skumapping
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    protected $logger;
    /**
     * @var Context
     */
    private $context;
    /**
     * @var Registry
     */
    private $coreRegistry;
    /**
     * @var LayoutFactory
     */
    private $resultLayoutFactory;
    /**
     * @var ConfigManager
     */
    private $configManager;
    /**
     * @var PageFactory
     */
    private $resultPageFactory;
    /**
     * @var McfLogManager
     */
    private $mcfLogManager;

    /**
     * @var Data
     */
    private $helper;

    /**
     * @var PublisherInterface
     */
    protected $publisher;
    /**
     * @var Json
     */
    private $json;

    /**
     * Construct function
     *
     * @param Context $context
     * @param Registry $coreRegistry
     * @param LayoutFactory $resultLayoutFactory
     * @param LoggerInterface $logger
     * @param PageFactory $resultPageFactory
     * @param ConfigManager $configManager
     * @param McfLogManager $mcfLogManager
     * @param PublisherInterface $publisher
     * @param Json $json
     * @param Data $helper
     */
    public function __construct(
        Context $context,
        Registry $coreRegistry,
        LayoutFactory $resultLayoutFactory,
        LoggerInterface $logger,
        PageFactory $resultPageFactory,
        ConfigManager $configManager,
        McfLogManager $mcfLogManager,
        PublisherInterface $publisher,
        Json $json,
        Data $helper
    ) {
        $this->configManager = $configManager;
        $this->resultPageFactory = $resultPageFactory;
        $this->mcfLogManager = $mcfLogManager;
        $this->helper = $helper;
        $this->publisher = $publisher;
        $this->json = $json;
        parent::__construct($context, $coreRegistry, $resultLayoutFactory, $logger, $configManager);
    }

    /**
     * Sku Mapping index action
     *
     * @return \Magento\Backend\Model\View\Result\Forward
     */
    public function execute()
    {
        $resultPage = $this->resultPageFactory->create();
        $resultPage->setActiveMenu('Amazonfulfillment_AmazonMCF::sku_mapping');
        $resultPage->getConfig()->getTitle()->prepend(__('SKU Mapping'));
        try {
            $activeWebsites = $this->helper->getActiveConnections();
            $msg = "";
            foreach ($activeWebsites as $activeWebsite) {
                $mcfItems = $this->configManager->getMcfRequest('inventory', [], $activeWebsite[0]);
                $data['mcfItems'] =  $mcfItems;
                $data['website'] =  $activeWebsite[0];
                if (isset($mcfItems['pagination']['nextToken'])) {
                    $data['nextToken'] = $mcfItems['pagination']['nextToken'];
                }
                $data = $this->json->serialize($data);
                $this->publisher->publish('amazonmcf.queue.mapping-automap', $data);
            }
            $this->messageManager->addSuccessMessage(__("Auto Map request is added to queue"));
            $msg = 'Auto Map request is added to queue';
            $logData = [
                'area' => __('AutoMap SKU'),
                'type' => __('Info'),
                'details' => $msg,
            ];
        } catch (\Exception $e) {
            $logData = [
                'area' => __('AutoMap SKU'),
                'type' => __('Error'),
                'details' => __("Failed to add to queue."),
            ];
            $this->messageManager->addErrorMessage(__("Failed to map SKUs automatically."));
        }
        if (isset($logData)) {
            $this->mcfLogManager->addLog($logData);
        }

        $resultRedirect = $this->resultRedirectFactory->create();
        return $resultRedirect->setPath('amazonmcf/skumapping/index', ['_current' => true]);
    }
}
