<?php declare(strict_types = 1);
/**
 * Copyright © 2022 Amazon. All rights reserved.
 */

namespace Amazonfulfillment\AmazonMCF\Model\Queue;

use Amazonfulfillment\AmazonMCF\Model\ConfigManager;
use Amazonfulfillment\AmazonMCF\Model\McfLogManager;
use Magento\Framework\Serialize\Serializer\Json;
use Amazonfulfillment\AmazonMCF\Logger\Logger;

class ConsumerAutomap
{
    /**
     * @var mapCount
     */
    public $mapCount = 0;
    
    /**
     * @var ConfigManager
     */
    private $configManager;
   
    /**
     * @var McfLogManager
     */
    private $mcfLogManager;
    /**
     * @var Logger
     */
    private $mcfLogger;
    /**
     * @var Json
     */
    private $json;

   /**
    * Construct function
    *
    * @param ConfigManager $configManager
    * @param McfLogManager $mcfLogManager
    * @param Logger $mcfLogger
    * @param Json $json
    */
    public function __construct(
        ConfigManager $configManager,
        McfLogManager $mcfLogManager,
        Logger $mcfLogger,
        Json $json
    ) {
        $this->configManager = $configManager;
        $this->mcfLogManager = $mcfLogManager;
        $this->mcfLogger = $mcfLogger;
        $this->json = $json;
    }

    /**
     * Process function
     *
     * @param string $data
     */
    public function process($data)
    {
        $this->mapCount = 0;
        $logData = [
            'user' => 'Cron Job',
            'area' => __('Auto Map SKU'),
            'type' => __('Info'),
            'details' => 'Auto Map queue running',
        ];
        if (isset($logData)) {
            $this->mcfLogManager->addLog($logData);
        }
        try {
            $details = $this->json->unserialize($data);
            $this->mcfLogger->notice('-AUTOMAP QUEUE PROCESS START-');
            if (isset($details['nextToken'])) {
                $dataToken['nextToken'] = $details['nextToken'];
            } else {
                $dataToken['nextToken'] = '';
            }
            $result = $this->mapSkus($details['mcfItems'], $dataToken, $details['website']);
            $this->mcfLogger->notice('-AUTOMAP QUEUE PROCESS END-');
            $logData = [
                'user' => 'Cron Job',
                'area' => __('Auto Map SKU'),
                'type' => __('Info'),
                'details' => 'Auto Map queue completed. <strong>'.$this->mapCount.'</strong> SKU(s) mapped/updated. 
                See detailed report in <strong>/var/log/amazonfulfillment_queue.log</strong>',
            ];
        } catch (\Exception $e) {
            $logData = [
                'user' => 'Cron Job',
                'area' => __('Auto Map SKU'),
                'type' => __('Info'),
                'details' => 'Auto Map queue error, See detailed  See detailed report in 
                <strong>/var/log/amazonfulfillment_queue.log</strong>',
            ];
        }
        if (isset($logData)) {
            $this->mcfLogManager->addLog($logData);
        }
    }
    /**
     * Map Skus function
     *
     * @param array $mcfItems
     * @param array $data
     * @param int $activeWebsite
     * @return void
     */
    public function mapSkus($mcfItems, $data, $activeWebsite)
    {
        if (isset($mcfItems['payload']['inventorySummaries'])) {
            $result = $this->updateMapping($mcfItems, $activeWebsite);
        }
        
        $mcfItems = $this->configManager->getMcfRequest('inventory', $data, $activeWebsite);
        $this->mcfLogger->notice('-Request to Fetch Seller SKUs from Amazon is Sent-');
        if (isset($mcfItems['pagination']['nextToken'])) {
            $data['nextToken'] = $mcfItems['pagination']['nextToken'];
            $this->mapSkus($mcfItems, $data, $activeWebsite);
        } else {
            $result = $this->updateMapping($mcfItems, $activeWebsite);
        }
    }
    /**
     * Update Mapping SKUS function
     *
     * @param array $mcfItems
     * @param int $activeWebsite
     * @return void
     */
    public function updateMapping($mcfItems, $activeWebsite)
    {
        $i = 0;
        $j = 0;
        try {
            if ($mcfItems != 404) {
                foreach ($mcfItems['payload']['inventorySummaries'] as $item) {
                    $mapArr['asin'] = $item['asin'];
                    $mapArr['seller_sku'] = $item['sellerSku'];
                    $mapArr['product_name'] = $item['productName'];
                    $mapArr['website'] = $activeWebsite;
                    $result = $this->configManager->skuMap($mapArr);
                
                    if (isset($result['status']) && ($result['status'] == 'saved' || $result['status'] == 'updated')) {
                        $this->mcfLogger->notice('Seller SKU '. $mapArr['seller_sku']." has been mapped");
                        $i++;

                    }
                    if (isset($result['status']) && $result['status'] == 'error') {
                        $this->mcfLogger->error('Seller SKU '. $mapArr['seller_sku']."  has error. See".$result['msg']);
                        $j++;
                    }
                }
            } else {
                $this->mcfLogger->error("Failed to map SKUs as no matching data found in MCF account");
            }
        } catch (\Exception $e) {
            $this->mcfLogger->error("Exception Error".$e->getMessage());
        }
        $this->mapCount = $this->mapCount + $i;
    }
}
