<?php declare(strict_types=1);
/**
 * Copyright © 2022 Amazon. All rights reserved.
 */
namespace Amazonfulfillment\AmazonMCF\Observer;
 
use Magento\Framework\Event\ObserverInterface;
use Amazonfulfillment\AmazonMCF\Model\FulfillmentOrderManager;
use Amazonfulfillment\AmazonMCF\Api\SkuMappingRepositoryInterface;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Magento\Store\Model\StoreManagerInterface;
use Amazonfulfillment\AmazonMCF\Helper\Data;

class OrderPlaceAfter implements ObserverInterface
{
    /**
     * @var FulfillmentOrderManager
     */
    protected $fulfillmentOrderManager;
    /**
     * @var SkuMappingRepositoryInterface
     */
    protected $skuMappingRepository;
    /**
     * @var SearchCriteriaBuilder
     */
    protected $searchCriteriaBuilder;
    /**
     * @var StoreManagerInterface
     */
    protected $storeManagerInterface;
    /**
     * @var Data
     */
    protected $helper;
    /**
     * @param FulfillmentOrderManager $fulfillmentOrderManager
     * @param SkuMappingRepositoryInterface $skuMappingRepository
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param StoreManagerInterface $storeManagerInterface
     * @param Data $helper
     */
    public function __construct(
        FulfillmentOrderManager $fulfillmentOrderManager,
        SkuMappingRepositoryInterface $skuMappingRepository,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        StoreManagerInterface $storeManagerInterface,
        Data $helper
    ) {
        $this->fulfillmentOrderManager = $fulfillmentOrderManager;
        $this->skuMappingRepository = $skuMappingRepository;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->storeManagerInterface = $storeManagerInterface;
        $this->helper = $helper;
    }
    /**
     * Execute
     *
     * @param Observer $observer
     * @return $this
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        $order = $observer->getEvent()->getOrder();
        $website = $this->storeManagerInterface->getStore()->getWebsiteId();
        $orderData = $this->fulfillmentOrderManager->getOrderItemsStatus($order);
        if ($this->helper->isConnectedExist($website, 'spapi_oauth_code') == '') {
            return $this;
        }
        if (!empty($orderData) && isset($orderData['amazon-mcf-status'])) {
            $order->setFulfilledByAmazon($orderData['amazon-mcf-status']);
            if ($orderData['amazon-mcf-status'] == 1 || $orderData['amazon-mcf-status'] == 2) {
                foreach ($order->getAllItems() as $item) {
                    if (isset($orderData['mcf_products']) &&
                    in_array($item->getProductId(), $orderData['mcf_products'])) {
                        $skuSearchCriteria = $this->searchCriteriaBuilder
                        ->addFilter('product_id', $item->getProductId(), 'eq')
                        ->addFilter('website', $website, 'eq')->create();
                        $skuMappingData = $this->skuMappingRepository->getList($skuSearchCriteria);
                        foreach ($skuMappingData->getItems() as $data) {
                            $item->setAmazonSellerSku($data['seller_sku']);
                        }
                        $item->setFulfilledByAmazon(1);
                    } else {
                        $item->setFulfilledByAmazon(0);
                    }
                }
            }
        }
        return $this;
    }
}
