<?php
/**
 * Copyright © 2022 Amazon. All rights reserved.
 */
declare(strict_types=1);

namespace Amazonfulfillment\AmazonMCF\Setup\Patch\Data;

use Magento\Eav\Setup\EavSetup;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;
use Amazonfulfillment\AmazonMCF\Setup\Patch\Data\McfAttributes;

class McfAttributeUpdate implements DataPatchInterface
{
   /** @var ModuleDataSetupInterface */
    private $moduleDataSetup;

   /** @var EavSetupFactory */
    private $eavSetupFactory;

   /**
    * @param ModuleDataSetupInterface $moduleDataSetup
    * @param EavSetupFactory $eavSetupFactory
    */
    public function __construct(
        ModuleDataSetupInterface $moduleDataSetup,
        EavSetupFactory $eavSetupFactory
    ) {
        $this->moduleDataSetup = $moduleDataSetup;
        $this->eavSetupFactory = $eavSetupFactory;
    }

   /**
    * @inheritdoc
    */
    public function apply()
    {
        /** @var EavSetup $eavSetup */
        $eavSetup = $this->eavSetupFactory->create(['setup' => $this->moduleDataSetup]);
        $eavSetup->updateAttribute(
            \Magento\Catalog\Model\Product::ENTITY,
            'mcf_enabled',
            [
                'is_visible' => '0',
                'group' => ''

            ]
        );
    }

   /**
    * @inheritdoc
    */
    public static function getDependencies()
    {
        return [
            McfAttributes::class
        ];
    }

   /**
    * @inheritdoc
    */
    public function getAliases()
    {
        return [];
    }

   /**
    * @inheritdoc
    */
    public static function getVersion()
    {
        return '1.0.2';
    }
}
