<?php
/**
 * Copyright © 2022 Amazon. All rights reserved.
 */
declare(strict_types=1);

namespace Amazonfulfillment\AmazonMCF\Setup\Patch\Data;

use Magento\Eav\Setup\EavSetup;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\Patch\DataPatchInterface;

class McfAttributes implements DataPatchInterface
{
   /** @var ModuleDataSetupInterface */
    private $moduleDataSetup;

   /** @var EavSetupFactory */
    private $eavSetupFactory;

   /**
    * @param ModuleDataSetupInterface $moduleDataSetup
    * @param EavSetupFactory $eavSetupFactory
    */
    public function __construct(
        ModuleDataSetupInterface $moduleDataSetup,
        EavSetupFactory $eavSetupFactory
    ) {
        $this->moduleDataSetup = $moduleDataSetup;
        $this->eavSetupFactory = $eavSetupFactory;
    }

   /**
    * @inheritdoc
    */
    public function apply()
    {
        /** @var EavSetup $eavSetup */
        $eavSetup = $this->eavSetupFactory->create(['setup' => $this->moduleDataSetup]);
        $eavSetup->addAttribute(\Magento\Catalog\Model\Product::ENTITY, 'mcf_enabled', [
                'type' => 'int',
                'backend' => '',
                'frontend' => '',
                'label' => 'MCF Mapped Status',
                'input' => 'boolean',
                'class' => '',
                'source' => '',
                'global' => \Magento\Eav\Model\Entity\Attribute\ScopedAttributeInterface::SCOPE_WEBSITE,
                'visible' => true,
                'required' => false,
                'user_defined' => false,
                'searchable' => false,
                'filterable' => false,
                'comparable' => false,
                'visible_on_front' => false,
                'used_in_product_listing' => false,
                'unique' => false,
                'sort_order' => 110,
                'group' => 'MCF',
                'apply_to' => ''
        ]);
    }

   /**
    * @inheritdoc
    */
    public static function getDependencies()
    {
        return [];
    }

   /**
    * @inheritdoc
    */
    public function getAliases()
    {
        return [];
    }

   /**
    * @inheritdoc
    */
    public static function getVersion()
    {
        return '1.0.0';
    }
}
