<?php
declare(strict_types=1);

/**
 * Copyright © 2022 Amazon. All rights reserved.
 */

namespace Amazonfulfillment\AmazonMCF\Ui\Component\Listing\Column;
 
use Magento\Framework\View\Element\UiComponent\ContextInterface;
use Magento\Framework\View\Element\UiComponentFactory;
use Magento\Ui\Component\Listing\Columns\Column;
use Magento\Catalog\Model\ProductRepository;

class MapStatus extends Column
{
    /**
     *
     * @param ContextInterface   $context
     * @param UiComponentFactory $uiComponentFactory
     * @param ProductRepository  $productRepository
     * @param array              $components
     * @param array              $data
     */
    public function __construct(
        ContextInterface $context,
        UiComponentFactory $uiComponentFactory,
        ProductRepository  $productRepository,
        array $components = [],
        array $data = []
    ) {
        parent::__construct($context, $uiComponentFactory, $components, $data);
        $this->productRepository = $productRepository;
    }
    /**
     * Prepare Data Source
     *
     * @param array $dataSource
     * @return array
     */
    public function prepareDataSource(array $dataSource)
    {
        foreach ($dataSource['data']['items'] as &$items) {
            $items['mcf_enabled'] = $this->getStatus($items['entity_id'], $items['mcf_website']);
        }
        return $dataSource;
    }
    /**
     * GetStatus
     *
     * @param int $productId
     * @param int $storeId
     * @return string
     */
    public function getStatus($productId, $storeId)
    {
        $productData = $this->productRepository->getById($productId, false, $storeId);
        if ($myCustomAttribute = $productData->getCustomAttribute('mcf_enabled')) {
            $status = $myCustomAttribute->getValue();
        } else {
            $status = '0';
        }

        if ($status == 1) {
            return __('Mapped');
        } else {
            return __('Not Mapped');
        }
    }
}
