<?php
declare(strict_types=1);

/**
 * Copyright © 2022 Amazon. All rights reserved.
 */

namespace Amazonfulfillment\AmazonMCF\Ui\Component\Listing\Column;
 
use Magento\Framework\View\Element\UiComponent\ContextInterface;
use Magento\Framework\View\Element\UiComponentFactory;
use Magento\Ui\Component\Listing\Columns\Column;
use Magento\Framework\Api\SearchCriteriaBuilder;
use Amazonfulfillment\AmazonMCF\Api\SkuMappingRepositoryInterface;
use Amazonfulfillment\AmazonMCF\Model\SkuMappingFactory;

class ProductName extends Column
{
    /**
     * @var ContextInterface
     */
    protected $context;
    /**
     * @var UiComponentFactory
     */
    protected $uiComponentFactory;
    /**
     * @var SearchCriteriaBuilder
     */
    private $searchCriteriaBuilder;
    /**
     * @var SkuMappingRepositoryInterface
     */
    private $skuMappingRepositoryInterface;
    /**
     * @var SkuMappingFactory
     */
    private $skuMappingFactory;
    /**
     * @var ConfigManager
     */
    private $configManager;
    /**
     * @param ContextInterface $context
     * @param UiComponentFactory $uiComponentFactory
     * @param SearchCriteriaBuilder $searchCriteriaBuilder
     * @param SkuMappingRepositoryInterface $skuMappingRepositoryInterface
     * @param SkuMappingFactory $skuMappingFactory
     * @param array $components
     * @param array $data
     */
    public function __construct(
        ContextInterface $context,
        UiComponentFactory $uiComponentFactory,
        SearchCriteriaBuilder $searchCriteriaBuilder,
        SkuMappingRepositoryInterface $skuMappingRepositoryInterface,
        SkuMappingFactory $skuMappingFactory,
        array $components = [],
        array $data = []
    ) {
        $this->searchCriteriaBuilder=$searchCriteriaBuilder;
        $this->skuMappingRepositoryInterface=$skuMappingRepositoryInterface;
        $this->skuMappingFactory=$skuMappingFactory;
        parent::__construct($context, $uiComponentFactory, $components, $data);
    }
 
    /**
     * Prepare Data Source
     *
     * @param array $dataSource
     * @return array
     */
    public function prepareDataSource(array $dataSource)
    {
        if (isset($dataSource['data']['items'])) {
            $fieldName = $this->getData('name');
            foreach ($dataSource['data']['items'] as & $item) {
                $skuMapping = $this->skuMappingFactory->create()->load($item['entity_id']);
                foreach ($skuMapping as $data) {
                    if (isset($data['amazon_product_name'])) {
                        $item[$fieldName] = $data['amazon_product_name'];
                    } else {
                        $item[$fieldName] = "";
                    }
                }
            }
        }
        return $dataSource;
    }
}
